import os
import time

import numpy as np
import pydicom
import torch

from mu_map.data.prepare import headers
from mu_map.data.remove_bed import add_bed
from mu_map.dataset.default import MuMapDataset
from mu_map.dataset.normalization import GaussianNormTransform
from mu_map.dataset.transform import SequenceTransform, PadCropTranform
from mu_map.file.dicom import load_dcm, update_dcm, change_uid
from mu_map.models.unet import UNet

torch.set_grad_enabled(False)

device = torch.device("cuda")
split_name = "test"

dir_train = "results/cgan_random_search_02/03"
dir_data = "data/second/"
dir_out = "results/mu_map_syn/"

file_weights = os.path.join(dir_train, "snapshots", "val_min_generator.pth")

model = UNet()
model.load_state_dict(torch.load(file_weights, map_location=device))
model = model.to(device)
model = model.eval()

transform_normalization = SequenceTransform([GaussianNormTransform(), PadCropTranform(dim=3, size=32)])
dataset = MuMapDataset(dir_data, transform_normalization=transform_normalization, split_name=split_name)
dataset_with_bed = MuMapDataset(dir_data, transform_normalization=PadCropTranform(dim=3, size=32), split_name=split_name, bed_contours_file=None)

for i, ((recon_nac, _), (_, mu_map)) in enumerate(zip(dataset, dataset_with_bed)):
    row = dataset.table.iloc[i]
    _id = row[headers.id]
    print(f"Process {_id} ...")

    recon_nac = recon_nac.to(device)
    mu_map_syn = model(recon_nac.unsqueeze(dim=0))

    mu_map = mu_map.squeeze().cpu().numpy()

    mu_map_syn = mu_map_syn.squeeze().cpu().numpy()
    mu_map_syn = np.where(mu_map_syn < 0.0, 0.0, mu_map_syn)
    mu_map_syn = add_bed(mu_map_syn, mu_map, dataset.bed_contours[_id])

    file_mu_map = os.path.join(dir_data, "images", row[headers.file_mu_map])
    dcm_mu_map = pydicom.dcmread(file_mu_map)

    dcm_mu_map_syn = update_dcm(dcm_mu_map, mu_map_syn)
    dcm_mu_map_syn = change_uid(dcm_mu_map_syn)

    base, ext = os.path.splitext(os.path.basename(file_mu_map))
    file_mu_map_syn = os.path.join(dir_out, f"{base}_syn{ext}")
    pydicom.dcmwrite(file_mu_map_syn, dcm_mu_map_syn)

